import { Component, OnInit, ViewChild, ElementRef } from '@angular/core';
import * as XLSX from 'xlsx';
import { ComplianceUploadReportService } from '../services/compliance-upload-report.service';


@Component({
  selector: 'app-compliance-upload-report',
  templateUrl: './compliance-upload-report.component.html',
  styleUrls: ['./compliance-upload-report.component.css']
})
export class ComplianceUploadReportComponent implements OnInit {
  @ViewChild('complianceReportTable') complianceReportTable: ElementRef;

  selectedFileFormat: string = 'excel'; // Default file format
  records: any[];

  constructor(private complianceUploadReportService: ComplianceUploadReportService) { }

  ngOnInit(): void {
    this.loadRecords();
  }

  loadRecords() {
    this.complianceUploadReportService.getAllComplianceUploadReportRecords().subscribe({
      next: (res: any) => {
        this.records = res;
      },
      error: (err) => {
        console.error('Error fetching compliance upload records:', err);
      }
    });
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.complianceReportTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Compliance_Report_${Date.now()}.${this.selectedFileFormat === 'excel' ? 'xlsx' : 'csv'}`;
    XLSX.writeFile(wb, fileName);
  }
}
