import { Component, ElementRef, ViewChild } from '@angular/core';
import * as XLSX from 'xlsx';
import { HighValueTransactionReportService } from '../services/high-value-transaction-report.service';

@Component({
  selector: 'app-high-value-transaction-report',
  templateUrl: './high-value-transaction-report.component.html',
  styleUrls: ['./high-value-transaction-report.component.css'],
})
export class HighValueTransactionReportComponent {
  @ViewChild('highValueTransactionTable')
  highValueTransactionTable: ElementRef;

  transactionNo: any = '';
  fromDate: any;
  toDate: any;
  allHighValueTransactions: any = [];
  selectedFileFormat: any = '1';

  constructor(
    private highValueTransactionSvc: HighValueTransactionReportService
  ) {}

  ngOnInit(): void {}

  filterByTransactioNo() {
    if (!this.transactionNo) {
      alert('Please Enter Transaction No!');
      return;
    }

    this.highValueTransactionSvc
      .getDataByTransactionNo(this.transactionNo)
      .subscribe({
        next: (res: any) => {
          this.allHighValueTransactions = res;
          this.transformData();
        },
        error: (err) => {
          this.allHighValueTransactions = [];
          alert(err?.error?.message || 'Something went wrong');
        },
        complete: () => {},
      });
  }

  displayByDate() {
    if (!this.fromDate && !this.toDate) {
      alert('Please Select From & To Date!');
      return;
    }

    this.highValueTransactionSvc
      .getFillGrid(this.fromDate, this.toDate)
      .subscribe({
        next: (res: any) => {
          this.allHighValueTransactions = res;
          this.transformData();
        },
        error: (err) => {
          this.allHighValueTransactions = [];
          alert(err?.error?.message || 'Something went wrong');
        },
        complete: () => {},
      });
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.highValueTransactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `View_High_Value_Transactions${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  transformData() {
    this.allHighValueTransactions = this.allHighValueTransactions.map(
      (item) => {
        const newItem = {};
        for (const key in item) {
          if (item.hasOwnProperty(key)) {
            const newKey = key.replace(/^\*|(?<=,)\*|\*(?=,|$)|\s+/g, ''); // Remove asterisk and replace spaces with underscores
            newItem[newKey] = item[key];
          }
        }
        return newItem;
      }
    );
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
