import { Component, ViewChild, TemplateRef } from '@angular/core';
import { BsModalService, BsModalRef } from 'ngx-bootstrap/modal';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ManageAgentUsersService } from '../services/manage-agent-users.service';
import { count } from 'rxjs';
import { AnyCatcher } from 'rxjs/internal/AnyCatcher';

@Component({
  selector: 'app-manage-agent-users',
  templateUrl: './manage-agent-users.component.html',
  styleUrls: ['./manage-agent-users.component.css'],
})
export class ManageAgentUsersComponent {
  bsModalRef: BsModalRef;
  manageCurrencyForm: FormGroup;

  @ViewChild('manageagentuser') manageAgentUserTemplate: TemplateRef<any>;
  @ViewChild('manageagentuserlogindetails')
  manageAgentUserLoginDetailsTemplate: TemplateRef<any>;
  @ViewChild('setDetails') setDetailsTemplate: TemplateRef<any>;

  countryList: any = [];
  CountryID: any;
  addGroupList: any;
  agentBranchId: any;
  gridGroupList: any;
  selectedCountry: any = '0';
  agentList: any = [];
  selectedAgent: any = '0';
  agentBranchList: any = [];
  selectedAgentBranch: any = '0';
  groupList: any = [];
  selectedGroup: any = '0';
  userList: any = [];
  selectedUser: any = '0';
  allUserRights: any = [];
  userNameId: any;
  AgentType: any;
  addCountryList: any;
  AgentId: any;
  addAgentList: any;
  AgentBranch: any;
  subGridGroupList: any;
  LoginDetails: any;
  setAgentUserDetails: any = [];
  agentUserBranch: any = {};

  constructor(
    private modalService: BsModalService,
    private fb: FormBuilder,
    private agentUserSvc: ManageAgentUsersService
  ) {
    this.createForm();
  }

  ngOnInit(): void {
    this.getCountryList();
  }

  createForm() {
    this.manageCurrencyForm = this.fb.group({
      user_name: ['', Validators.required],
      email: ['', [Validators.required, Validators.email]],
      password: ['', Validators.required],
      BranchCode: ['', Validators.required],
      AgentTypeID: ['None', Validators.required],
      CountryID: ['None', Validators.required],
      AgentId: ['None', Validators.required],
      AgentBranchID: ['None', Validators.required],
      SubUserGroup_Id: ['None', Validators.required],
      EmailID: ['', Validators.required],
    });
  }

  openManageAgentUserModal() {
    this.manageCurrencyForm.reset({
      BranchCode: '',
      AgentTypeID: 'None',
      CountryID: 'None',
      AgentId: 'None',
      AgentBranchID: 'None',
      SubUserGroup_Id: 'None',
      EmailID: '',
    });

    this.addAgentType();
    this.bsModalRef = this.modalService.show(this.manageAgentUserTemplate, {
      class: 'modal-lg',
    });
  }

  addUser() {
    this.add();
    this.bsModalRef.hide();
  }

  getCountryList() {
    this.agentUserSvc.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId) {
    if (countryId != '0') {
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0) {
    this.agentUserSvc.getAgentList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
      },
      error: (err) => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  onAgentChange(agentId) {
    if (agentId != '0' && this.selectedCountry != '0') {
      this.getAgentBranchList(this.selectedCountry, agentId);
    }
  }

  getAgentBranchList(countryId: any = 0, agentId: any = 0) {
    this.agentUserSvc.getAgentBranchList(countryId, agentId).subscribe({
      next: (res: any) => {
        this.agentBranchList = res;
      },
      error: (err) => {
        this.agentBranchList = [];
      },
      complete: () => {},
    });
  }

  onAgentBranchChange(agentBranchId) {
    this.agentBranchId = agentBranchId;
    if (
      agentBranchId != '0' &&
      this.selectedCountry != '0' &&
      this.selectedAgent != '0'
    ) {
      this.getGroupList(
        this.selectedCountry,
        this.selectedAgent,
        agentBranchId
      );
    }
  }

  getGroupList(countryId: any = 0, agentId: any = 0, agentBranchId: any = 0) {
    this.agentUserSvc
      .getGroupList(countryId, agentId, agentBranchId)
      .subscribe({
        next: (res: any) => {
          this.groupList = res;
        },
        error: (err) => {
          this.groupList = [];
        },
        complete: () => {},
      });
  }

  onGroupChange(groupId) {
    if (
      groupId != '0' &&
      this.selectedCountry != '0' &&
      this.selectedAgent != '0' &&
      this.selectedAgentBranch != '0'
    ) {
      this.getUserList(
        this.selectedCountry,
        this.selectedAgent,
        this.selectedAgentBranch,
        groupId
      );
    }
  }

  getUserList(
    countryId: any = 0,
    agentId: any = 0,
    agentBranchId: any = 0,
    groupId: any = 0
  ) {
    this.agentUserSvc
      .getUserList(countryId, agentId, agentBranchId, groupId)
      .subscribe({
        next: (res: any) => {
          this.userList = res;
        },
        error: (err) => {
          this.userList = [];
        },
        complete: () => {},
      });
  }

  filter() {
    if (
      this.selectedAgent != '0' &&
      this.selectedAgentBranch != '0' &&
      this.selectedGroup != '0' &&
      this.selectedUser != '0'
    ) {
      this.agentUserSvc
        .getFillGrid(
          this.selectedAgent,
          this.selectedAgentBranch,
          this.selectedGroup,
          this.selectedUser
        )
        .subscribe({
          next: (res: any) => {
            this.allUserRights = res;
            this.gridGroup(
              this.allUserRights[0].Agent_ID,
              this.allUserRights[0].User_Group
            );
            this.subGridGroup(this.allUserRights[0].User_Group);
          },
          error: (err) => {
            this.allUserRights = [];
          },
          complete: () => {},
        });
    }
  }

  getDetails() {
    this.agentUserSvc.getDetails(1, 1, this.userNameId).subscribe({
      next: (res: any) => {
        this.allUserRights = res[0].manageAgentUserResponse;
        this.allUserRights = this.allUserRights.concat(
          res[0]?.manageAgentUserDetailResponse
        );
        this.gridGroup(
          this.allUserRights[0].Agent_ID,
          this.allUserRights[0].User_Group
        );
        this.subGridGroup(this.allUserRights[0].User_Group);
      },

      error: (err) => {},
    });
  }

  addAgentType() {
    this.agentUserSvc.addAgentType().subscribe({
      next: (res: any) => {
        this.AgentType = res;
      },
      error: (err) => {},
    });
  }

  onSelectAddAgent(AddAgentId) {
    this.AgentId = AddAgentId;
    this.addCountry(AddAgentId);
    this.addGroup(AddAgentId, 1);
  }

  addCountry(AddAgentId: any) {
    this.agentUserSvc.addCountry(AddAgentId).subscribe({
      next: (res: any) => {
        this.addCountryList = res;
      },
      error: (err) => {},
    });
  }

  onSelectAddCountry(CountryID) {
    this.CountryID = CountryID;
    this.addAgent(CountryID);
  }

  addAgent(CountryID) {
    this.agentUserSvc.addAgent(CountryID, this.AgentId).subscribe({
      next: (res: any) => {
        this.addAgentList = res;
      },
      error: (err) => {},
    });
  }

  onSelectAgent(AgentID) {
    this.addAgentBranch(AgentID);
  }

  addAgentBranch(AgentID) {
    this.agentUserSvc.addAgentBranch(AgentID).subscribe({
      next: (res: any) => {
        this.AgentBranch = res;
        console.log('res: ', res);
      },
      error: (err) => {},
    });
  }

  addGroup(AgentTypeID: any, G_UserGroup: any) {
    this.agentUserSvc.addGroup(AgentTypeID, G_UserGroup).subscribe({
      next: (res: any) => {
        this.addGroupList = res;
      },
      error: (err) => {},
    });
  }

  searchByBranchCode(BranchCode: any) {
    this.agentUserSvc.searchByBranchCode(1, 1, this.f.BranchCode).subscribe({
      next: (res: any) => {
        console.log('res: ', res);
        this.manageCurrencyForm.patchValue({
          AgentTypeID: res[0].AgentTypeID,
        });
        this.addCountry(res[0].AgentTypeID);
        this.manageCurrencyForm.patchValue({
          CountryID: res[0].CountryID,
        });
        this.manageCurrencyForm.patchValue({
          CountryID: res[0].CountryID,
        });

        this.agentUserSvc
          .addAgent(res[0].CountryID, res[0].AgentTypeID)
          .subscribe({
            next: (res: any) => {
              this.addAgentList = res;
            },
            error: (err) => {},
          });

        this.manageCurrencyForm.patchValue({
          AgentId: res[0].AgentID,
        });
        this.addAgentBranch(res[0].AgentID);

        this.manageCurrencyForm.patchValue({
          AgentBranchID: res[0].AgentBranchID,
        });
        this.addGroup(res[0].AgentTypeID, 1);
      },
      error: (err) => {},
    });
  }

  add() {
    const payload = {
      UserName: 'None',
      AgentTypeID: this.f.AgentTypeID,
      GroupID: this.f.SubUserGroup_Id,
      G_Branch_ID: this.f.AgentBranchID,
      AgentID: this.f.AgentId,
      AgentBranchID: 1,
      EmailID: String(this.f.EmailID),
      G_UserGroup: 1,
      IsCheckUserTime: true,
      G_UserID: 1,
    };
    this.agentUserSvc.add(payload).subscribe({
      next: (res: any) => {
        res;
      },
      error: (err) => {},
    });
  }

  gridGroup(AgentID, UserGroup) {
    this.agentUserSvc.gridGroup(AgentID, UserGroup).subscribe({
      next: (res: any) => {
        this.gridGroupList = res;
      },
      error: (err) => {},
    });
  }

  subGridGroup(User_Group) {
    this.agentUserSvc.subGridGroup(User_Group).subscribe({
      next: (res: any) => {
        this.subGridGroupList = res;
      },
      error: (err) => {},
    });
  }

  getLoginDetails(UserID) {
    this.bsModalRef = this.modalService.show(
      this.manageAgentUserLoginDetailsTemplate
    );
    this.agentUserSvc.getLoginDetails(UserID).subscribe({
      next: (res: any) => {
        this.LoginDetails = res;
      },
      error: (err) => {},
    });
  }

  isDeleted() {
    return this.allUserRights
      .filter((con) => con.isDeleted)
      .map((con) => ({
        UserID: this.allUserRights[0].UserId,
        UserName: this.allUserRights[0].User_Name,
        IsChkDelete: this.allUserRights[0].isDeleted,
      }));
  }

  Delete() {
    const payload = this.isDeleted();
    console.log('payload: ', payload);
    this.agentUserSvc.Delete(payload).subscribe({
      next: (res: any) => {
        res;
        console.log(' res;: ', res);
      },
      error: (err) => {},
    });
  }

  update() {
    if (this.allUserRights[0].isDeleted) {
      this.Delete();
      return;
    }
    if (!this.allUserRights[0].isDeleted) {
      const payload = [
        {
          UserID: String(this.allUserRights[0].UserId),
          UserName: String(this.allUserRights[0].User_Name),
          EmailID: String(this.allUserRights[0].EmailID) || 'None',
          GroupID: this.allUserRights[1]?.GroupID || 0,
          SubGroupID: this.allUserRights[0].SubUserGroup_Id,
          Locked: String(this.allUserRights[0].Locked),
          IsChkReset: true,
          Firstlogin: String(this.allUserRights[0].FirstLogin),
          WebLogin: String(this.allUserRights[0].AllowWebLogin),
          G_UserID: String(this.allUserRights[0].User_Group),
        },
      ];
      console.log('payload: ', payload);
      // this.agentUserSvc.update(payload).subscribe({
      //   next: (res: any) => {
      //     res;
      //     console.log(' res;: ', res);
      //   },
      //   error: (err) => {},
      // });
    }
  }
  addUserForSameLoction() {
    this.openManageAgentUserModal();
    this.agentUserSvc.addUserForSameLoction(this.agentBranchId).subscribe({
      next: (res: any) => {
        this.manageCurrencyForm.patchValue({
          AgentTypeID: res[0].AgentTypeID,
        });
        this.addCountry(res[0].AgentTypeID);
        this.manageCurrencyForm.patchValue({
          CountryID: res[0].CountryID,
        });
        this.manageCurrencyForm.patchValue({
          CountryID: res[0].CountryID,
        });

        this.agentUserSvc
          .addAgent(res[0].CountryID, res[0].AgentTypeID)
          .subscribe({
            next: (res: any) => {
              this.addAgentList = res;
            },
            error: (err) => {},
          });

        this.manageCurrencyForm.patchValue({
          AgentId: res[0].AgentID,
        });
        this.addAgentBranch(res[0].AgentID);

        this.manageCurrencyForm.patchValue({
          AgentBranchID: res[0].AgentBranchID,
        });
        this.addGroup(res[0].AgentTypeID, 1);
      },
      error: (err) => {},
    });
  }

  get f() {
    return this.manageCurrencyForm.value;
  }
  openSetDetailsModal() {
    this.bsModalRef = this.modalService.show(this.setDetailsTemplate, {
      class: 'modal-lg',
    });
  }

  setAgentUser(agentUser: any) {
    if (agentUser) {
      let payload;
      payload = {
        UserID: agentUser?.UserId,
        GroupID: agentUser?.User_Group,
        SubGroupID: agentUser?.SubUserGroup_Id,
        G_UserGroupID: 1,
      };
      this.agentUserSvc.setAgentUser(payload).subscribe({
        next: (res: any) => {
          this.setAgentUserDetails = res;
          this.agentUserBranch = res?.[0]?.ManageAgentUserBranchResponse?.[0];
          this.openSetDetailsModal();
        },
        error: (err) => {},
        complete: () => {},
      });
    }
  }

  setTree() {
    let payload;
    payload = [
      {
        GroupID:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].Group_Id,
        SubGroupID:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].SubUserGroup_Id,
        UserID: this.agentUserBranch?.UserID,
        FunctionID:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].FunctionID,
        FirstFunctionID:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].FirstFunctionID,
        SecondFunctionID:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].SecondFunctionID,
        ThirdFunctionID:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].ThirdFunctionID,
        FourthFunctionID:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].FourthFunctionID,
        IschkLevel1:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].FirstActive,
        IschkLevel1Visible: true,
        IschkLevel2:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].SecondActive,
        IschkLevel2Visible: true,
        IschkLevel3:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].ThirdActive,
        IschkLevel3Visible: true,
        IschkLevel4:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].FourthActive,
        IschkLevel4Visible: true,
        IschkAccess_Add:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].Access_Add,
        IschkAccess_AddVisible:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].Add_IsVisible,
        IschkAccess_Edit:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].Access_Edit,
        IschkAccess_EditVisible:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].Edit_IsVisible,
        IschkAccess_Delete:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].Access_Delete,
        IschkAccess_DeleteVisible:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].Delete_IsVisible,
        IschkAccess_View:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].Access_View,
        IschkAccess_ViewVisible:
          this.setAgentUserDetails?.[0]
            ?.ManageAgentUserPopulateGroupResponse?.[0].View_IsVisible,
      },
    ];
    this.agentUserSvc.setAgentUser(payload).subscribe({
      next: (res: any) => {
        this.bsModalRef.hide();
      },
      error: (err) => {},
      complete: () => {
        this.bsModalRef.hide();
      },
    });
  }
}
