import { Component, TemplateRef, ViewChild } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { ManageSourceOfIncomeService } from '../services/manage-source-of-income.service';

@Component({
  selector: 'app-manage-source-of-income',
  templateUrl: './manage-source-of-income.component.html',
  styleUrls: ['./manage-source-of-income.component.css'],
})
export class ManageSourceOfIncomeComponent {
  @ViewChild('sourceOfIncome') sourceOfIncomeModal;
  bsModalRef: BsModalRef;

  sourceOfIncomeList: any = [];
  selectedIncome: any = '0';
  allSourceOfIncome: any = [];
  type: any = 'add';

  sourceOfIncomeForm = new FormGroup({
    SourceOfIncome_ID: new FormControl(),
    SourceOfIncomeName: new FormControl('', Validators.required),
    chkIsActive: new FormControl(false),
  });

  constructor(
    private sourceOfIncomeSvc: ManageSourceOfIncomeService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getSourceOfIncomeList();
    this.getFillGrid();
  }

  getSourceOfIncomeList() {
    this.sourceOfIncomeSvc.getSourceOfIncomeList().subscribe({
      next: (res: any) => {
        this.sourceOfIncomeList = res;
      },
      error: (err) => {
        this.sourceOfIncomeList = [];
      },
      complete: () => {},
    });
  }

  getFillGrid(sourceOfIncome: any = 0) {
    this.sourceOfIncomeSvc.getFillGrid(sourceOfIncome).subscribe({
      next: (res: any) => {
        this.allSourceOfIncome = res;
        if (this.allSourceOfIncome.length) {
          this.allSourceOfIncome = this.allSourceOfIncome.flatMap(
            (item) => item.incomeDetails
          );
        }
      },
      error: (err) => {
        this.allSourceOfIncome = [];
      },
      complete: () => {},
    });
  }

  filter() {
    if (this.selectedIncome) {
      this.getFillGrid(this.selectedIncome);
    }
  }

  addSourceOfIncome() {
    if (!this.sourceOfIncomeForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    let serviceObservable;

    if (this.type === 'edit') {
      serviceObservable = this.sourceOfIncomeSvc.updateSourceOfIncome(
        this.f.SourceOfIncome_ID,
        this.f.SourceOfIncomeName,
        this.f.chkIsActive ? this.f.chkIsActive : false
      );
    } else {
      serviceObservable = this.sourceOfIncomeSvc.addSourceOfIncome(
        this.f.SourceOfIncomeName,
        this.f.chkIsActive ? this.f.chkIsActive : false
      );
    }

    serviceObservable.subscribe({
      next: (res: any) => {
        this.getFillGrid();
        this.modalService.hide();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openSourceOfIncomeModal(type: any) {
    if (type == 'add') {
      this.sourceOfIncomeForm.reset();
    }
    this.type = type;
    this.openModal(this.sourceOfIncomeModal);
  }

  get f() {
    return this.sourceOfIncomeForm.value;
  }

  reset() {
    this.sourceOfIncomeForm.reset();
  }

  editSourceofIncome(sourceOfIncome: any) {
    this.sourceOfIncomeForm.patchValue({
      SourceOfIncome_ID: sourceOfIncome?.SourceOfIncome_ID,
      SourceOfIncomeName: sourceOfIncome?.SourceOfIncome,
      chkIsActive: sourceOfIncome?.IsActive,
    });
    this.openSourceOfIncomeModal('edit');
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
