import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class ViewComplianceListService {
  url = environment.apiUrl;

  constructor(private http: HttpClient) {}

  getFillGrid(transactionID: any = -1) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/compliance-management/view?UserID=0&TransactionID=${transactionID}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getDataByTransactionNo(transactionNumber: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/compliance-management/viewby-transactionnumber?UserID=0&TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getViewComplianceByTransactionNo(transactionNumber: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/compliance-management/view-compliance?TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getTransactionLogByTransactionNo(transactionNumber: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/compliance-management/transaction-log?TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
 
  getViewByTransactionId(userId: any = 0, transactionId: any = '') {
    return this.http.get(
      `${environment.apiUrl}/v1/api/compliance-management/viewby-transactionid?UserID=${userId}&TransactionNumber=${transactionId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
  getTransactionPrefix(userId: any = 0, transactionNumber: any = '') {
    return this.http.get(
      `${environment.apiUrl}/v1/api/compliance-management/transaction-prefix?UserID=${userId}&TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getRequiredDetail(transactionNumber: any = '') {
    return this.http.get(
      `${environment.apiUrl}/v1/api/compliance-management/required-detail?TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getBindData(transactionNumber: any = '') {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/compliance-management/bind-data?TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getBindWithoutSbiTransactionNumber(transactionNumber: any = '') {
    return this.http.get(
      `${environment.apiUrl}/v1/api/compliance-management/bind-without-sbi-transactionnumber?TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
  getComplianceShowDetail(transactionNumber: any = '', searchType: any = '') {
    return this.http.get(
      `${environment.apiUrl}/v1/api/compliance-management/compliance-show-detail?TransactionNumber=${transactionNumber}&SearchType=${searchType}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  updateComplianceOnHold(
    transactionNumber: any = '',
    remark: any = '',
    userId: any = 1
  ) {
    const body = {};

    return this.http.patch(
      `${environment.apiUrl}/v1/api/compliance-management/onhold?TransactionNumber=${transactionNumber}&Remark=${remark}&UserID=${userId}`,
      body,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  updateCustomerBeneficiary(
    transactionNumber: any = '',
    isApprove: any = '',
    sType: any = '',
    remark: any = '',
    userId: any = '',
    customerBeneficiaryNo: any = ''
  ) {
    const body = {
      TransactionNumber: transactionNumber,
      IsApprove: isApprove,
      sType: sType,
      Remark: remark,
      UserID: userId,
      CustomerBeneficiaryNo: customerBeneficiaryNo,
    };

    return this.http.patch(
      `${environment.apiUrl}/v1/api/compliance-management/updatecustben`,
      body,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  approveCompliance(
    transactionNumber: any = '',
    remark: any = '',
    userId: any = ''
  ) {
    const body = {  };

    return this.http.patch(
      `${environment.apiUrl}/v1/api/compliance-management/approve?TransactionNumber=${transactionNumber}&Remark=${remark}&UserID=${userId}`,
      body,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  rejectCompliance(
    transactionNumber: any = '',
    isMGTxn: any = '',
    isMTOSuccess: any = '',
    rejectReason: any = '',
    userId: any = '',
    orgMsgID: any = ''
  ) {
    return this.http.get(
      `${environment.apiUrl}/v1/api/compliance-management/reject?TransactionNumber=${transactionNumber}&IsMGTxn=${isMGTxn}&IsMTOSuccess=${isMTOSuccess}&RejectReason=${rejectReason}&UserID=${userId}&OrgMsgID=${orgMsgID}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }






    viewByTransactionId(transactionNumber: any, showOFAC: any) {
      return this.http.get(
        `${environment.apiUrl}/v1/api/complicance/view-complaince-hold-list/view`,
        {
          params: {
            TransactionNumber: transactionNumber,
            showOFAC: showOFAC,
          },
          headers: {
            Authorization: getCacheData('admin_token'),
            Accept: 'application/json',
            username: getCacheData('username'),
          },
        }
      );
    }
}